function [labels]=m_gen_mol_coord_rods(rod_coords, params)
% This function places the contact point on the surfaces of rod structures.
% These rods consists of a cylindrical surface closed with hemisphered at
% their ends.
% First calculates the number of contact points on the individual
% rods, then places the contact point on them uniformly and connects
% the linkers to the contact points. The base of the half sphere of the
% linkers has to be tangential to the rod's surface.

% PARENT: m_gen_mol_coord.m
% CHILDS: -

% if fixed rod parameters were chosen
if strcmp(params.sample.rod_sim_type,'fix')
    N_rod=size(params.sample.rod_fix_N,2);  % number of rods
    N_cont_rod=params.sample.rod_fix_N;  % number of contact point on the individual rods
    R_rod=params.sample.rod_fix_size(1,:);    % radius of the individual rods' cylindrical surfaces
    length_cyl=params.sample.rod_fix_size(2,:);  % length of the individual rods' cylindrical surfaces

    theta=rod_coords.theta;
    fi=rod_coords.fi;
    
% if random rod parameters were chosen
elseif strcmp(params.sample.rod_sim_type,'random')
    N_rod=params.sample.rod_N_ofrod;    % number of rods
    N_mean=params.sample.rod_N_ofmol;   % mean number of contact points on each rod
    N_var=params.sample.rod_Sig_ofmol;  % variance of the number of dye molecules on each rod
    P=1-N_var./N_mean;   % success probability in each trial
    
    % if the probability has negative or zero value...
    if P<=0
        P=0;        % count it as zero
        N_trials=0;
        disp('Too large variance was given for vesicules...')
    % if "N_mean" is zero
    elseif isnan(P)
        P=0;
        N_trials=0;
    % if P has meaningful value
    else
        N_trials=round(N_mean./P);    % number of trials in the binomial distribution
    end

    N_cont_rod=m_gen_random('binomial', N_trials, P, 1, N_rod);      % number of contact points on the individual rods
    
    R_rod=repmat(params.sample.rod_Size_ofrod(1), size(N_cont_rod));    % radius of the individual rods' cylindrical surfaces
    length_cyl=repmat(params.sample.rod_Size_ofrod(2), size(N_cont_rod));    % length of the individual rods' cylindrical surfaces
    % spherical angles for rod rotation, only rotated in the "x-y"
    % plane uniformly
    theta=rod_coords.theta;
    fi=rod_coords.fi;
    
else
    error('Error in vesicles parameters');
end




% calculating the number contact points on the rods hemispherical (o the caps) and cylindrical surfaces 
cap_surface=4*R_rod.^2*pi;
cyl_surface=2*R_rod*pi.*length_cyl;
P=cap_surface./(cap_surface+cyl_surface);
N_trials=N_cont_rod;
N_cont_rod_cap=zeros(size(N_cont_rod));
for idxTrial=1:size(N_trials,2)
    N_cont_rod_cap(1,idxTrial)=m_gen_random('binomial', N_trials(idxTrial), P(idxTrial), 1, 1);
end
N_cont_rod_cyl=N_cont_rod-N_cont_rod_cap;

% geting the initial label coordinates binded to the caps, calling the
% function for the vesicle label calculation
cap_params.sample.name='vesicles';
cap_params.dye.link_rad=params.dye.link_rad;
cap_params.dye.N_of_labels_per_epitope=params.dye.N_of_labels_per_epitope;
cap_params.dye.var_N_of_labels_per_epitope=params.dye.var_N_of_labels_per_epitope;
cap_params.dye.binding_angle=params.dye.binding_angle;
cap_params.dye.binding_angle_dev=params.dye.binding_angle_dev;

cap_labels.init_cont_coords=[];
cap_labels.init_dye_coords=[];
cap_labels.init_orientation=[];
for idxRod=1:N_rod
    cap_params.sample.ves_sim_type='fix';
    cap_params.sample.ves_fix_N=N_cont_rod_cap(idxRod);
    cap_params.sample.ves_ellipsoidBool=false;
    
    cap_point_coords.pos=zeros(3,1);
    cap_point_coords.ves_N=1;
    cap_point_coords.rad=R_rod(idxRod);
    cap_point_coords.fi=0;
    cap_point_coords.theta=0;
    
    [cap_labels_temp] = m_gen_mol_coord_vesicles(cap_point_coords, cap_params);
    
    % translating the got coordinates to place them at the both and the rod
    signBool=cap_labels_temp.init_cont_coords(1,:)>=0;
    cap_labels_temp.init_cont_coords(1,:)=cap_labels_temp.init_cont_coords(1,:)+(signBool-1/2)*length_cyl(idxRod);
    cap_labels_temp.init_dye_coords(1,:)=cap_labels_temp.init_dye_coords(1,:)+(signBool-1/2)*length_cyl(idxRod);
    
    cap_labels.init_cont_coords=[cap_labels.init_cont_coords, cap_labels_temp.init_cont_coords];
    cap_labels.init_dye_coords=[cap_labels.init_dye_coords, cap_labels_temp.init_dye_coords];
    cap_labels.init_orientation=[cap_labels.init_orientation, cap_labels_temp.init_orientation];
   
end

% geting the initial label coordinates binded to the cylindrical surface, calling the
% function for the microtubule label calculation 
cyl_params.sample.name='lines';
cyl_params.dye.link_rad=params.dye.link_rad;
cyl_params.dye.N_of_labels_per_epitope=params.dye.N_of_labels_per_epitope;
cyl_params.dye.var_N_of_labels_per_epitope=params.dye.var_N_of_labels_per_epitope;
cyl_params.dye.binding_angle=params.dye.binding_angle;
cyl_params.dye.binding_angle_dev=params.dye.binding_angle_dev;

cyl_labels.init_cont_coords=[];
cyl_labels.init_dye_coords=[];
cyl_labels.init_orientation=[];
for idxRod=1:N_rod
    cyl_vert_coords{1,1}=zeros(3,2);
    cyl_vert_coords{1,1}(1,1)=length_cyl(idxRod)/2;
    cyl_vert_coords{1,1}(1,2)=-length_cyl(idxRod)/2;
    
    cyl_params.sample.label_dens=N_cont_rod_cyl(idxRod)./length_cyl(idxRod)/1E-3;
    cyl_params.sample.lines_radius=R_rod(idxRod);
    
	[cyl_labels_temp] = m_gen_mol_coord_tubules(cyl_vert_coords, cyl_params);
    cyl_labels.init_cont_coords=[cyl_labels.init_cont_coords, cyl_labels_temp.init_cont_coords];
    cyl_labels.init_dye_coords=[cyl_labels.init_dye_coords, cyl_labels_temp.init_dye_coords];
    cyl_labels.init_orientation=[cyl_labels.init_orientation, cyl_labels_temp.init_orientation];
end

% concatenating the got coordinates
contact_point=[cap_labels.init_cont_coords, cyl_labels.init_cont_coords];  % variable containing contact point coordinates in 3XN_cont array
label_points=[cap_labels.init_dye_coords, cyl_labels.init_dye_coords];   % variable containing dye coordinates in 3xM_mol array
label_orientation=[cap_labels.init_orientation, cyl_labels.init_orientation];   % variable containing dye orientations (azimuth, polar angles) in 3xM_mol array


N_cont=sum(N_cont_rod); % N of contact points in the sample


% indexing, helps determining which contact point to which vesicule belongs
rod_indices=zeros(1,N_cont);
idx_low=1;
idx_up=0;
for idx_rod=1:N_rod
    idx_up=idx_up+N_cont_rod_cap(idx_rod);
    rod_indices(idx_low:idx_up)=idx_rod;
    idx_low=idx_up+1;
end
for idx_rod=1:N_rod
    idx_up=idx_up+N_cont_rod_cyl(idx_rod);
    rod_indices(idx_low:idx_up)=idx_rod;
    idx_low=idx_up+1;
end

% calculate molecule positions
for idx_cont=1:N_cont
    % get parameters of vesicle where the actual molecule is located
    idx_rod=rod_indices(idx_cont); %get index 
    pos_rod=rod_coords.pos(:,idx_rod); %get center position
    
    % rotation matrices of the rod orientations
    R_y=[cos(theta(idx_rod)),0,sin(theta(idx_rod));...
        0,1,0;...
        -sin(theta(idx_rod)),0,cos(theta(idx_rod))];
    R_z=[cos(fi(idx_rod)),-sin(fi(idx_rod)),0;...
        sin(fi(idx_rod)),cos(fi(idx_rod)),0;...
        0,0,1];
    
    % calculating the label coordinates and orientations and the contact
    % point coordinates
    labels.init_cont_coords(:,idx_cont)=R_z*R_y*contact_point(:,idx_cont)+pos_rod;  % variable containing contact point coordinates in 3XN_cont array
    labels.init_dye_coords(:,idx_cont)=R_z*R_y*label_points(:,idx_cont)+pos_rod;  % variable containing dye coordinates in 3xM_mol array
    labels.init_orientation(:,idx_cont)=label_orientation(:,idx_cont)+[fi(idx_rod);theta(idx_rod)];  % variable containing dye orientations (azimuth, polar angles) in 3xM_mol array
end

end