function [ vesicle_coords ] = m_gen_struct_vesicles( params )
% This function generates the center positions of vesicles. In case of
% user given coordinates it simply places the given coordinates in an
% array. Otherwise randomly places the vesicle centers in the given area.
% The overlapping of randomly placed vesicles are prevented, in case of of
% ellipsoidal vesicles the larget axis is used to achieve this.

% PARENT: m_gen_struct.m
% CHILDS: -

% if fixed coordinates were chosen
if strcmp(params.sample.ves_sim_type,'fix')
    vesicle_coords.pos=[...
        params.sample.ves_fix_x;...
        params.sample.ves_fix_y;...
        params.sample.ves_fix_z];
    
    R_ves=params.sample.ves_fix_rad;    % radius of the individual vesicules
    vesicle_coords.rad=R_ves;
    
    ves_N=size(params.sample.ves_fix_x,2);  % number of vesicles
    vesicle_coords.ves_N=ves_N;
    
    % rotation angles of the ellipsoid
    %theta_rot_ellipsoids=acos(1-2*rand(N_ves,1));     % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.theta=zeros(ves_N,1);      % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.fi=2*pi*rand(ves_N,1);  % angle for the ellipsoid's rotation around the "z" axis
    %fi_rot_ellipsoids=zeros(N_ves,1);  % angle for the ellipsoid's rotation around the "z" axis
    
    
% if random coordinates were chosen
elseif strcmp(params.sample.ves_sim_type,'random')
    %% vesicles on synaptosomes
    % these parameters must be set by hand: 
    synaptosome_posX=params.sample.ves_rand_synaptosomes_pos(1,:);      % "x" psoition of synaptosomes' centres
    synaptosome_posY=params.sample.ves_rand_synaptosomes_pos(2,:);      % "y" psoition of synaptosomes' centres
    synaptosome_posZ=params.sample.ves_rand_synaptosomes_pos(3,:);      % "z" psoition of synaptosomes' centres
    synaptosome_rad=params.sample.ves_rand_synaptosomes_rad;        % radius of each synaptosome
    synaptosome_vesN=params.sample.ves_rand_synaptosomes_N;    % number of vesicles in each synaptosome

    synaptosome_pos=[synaptosome_posX; synaptosome_posY; synaptosome_posZ]; % 3x'synaptosome number' array containing the synaptosome positions
    synaptosome_N=size(synaptosome_pos,2);      % number of synaptosomes
    
    ves_N_synaptosomes=sum(synaptosome_vesN);       % total number of vesicles in every synaptosome
    
    % initializing the structure array containing the synaptosome 
    % parameters
    synaptosome=repmat(struct('pos', [],'rad', [],'ves_N', [],'ves_pos', [],'ves_rad', []), [1,synaptosome_N]);
    
    % parameters describing the distribution of the vesicle radius
    R_mean_max=max(params.sample.ves_rand_rad_mean);   % mean radius of vesicles, largest axis in case of ellipsoidal vesicles
    R_mean=params.sample.ves_rand_rad_mean;   % mean radii of vesicles' axes
    R_sig=params.sample.ves_rand_rad_std;    % sigma of radius of vesicles
    
    % loop over the synaptosomes
    for idxSynap=1:synaptosome_N
        % parameters of the actual synaptosome
        synaptosome(idxSynap).pos=synaptosome_pos(:, idxSynap);     % position of the actual synaptosome
        synaptosome(idxSynap).rad=synaptosome_rad(1, idxSynap);     % radius of the actual synaptosome
        synaptosome(idxSynap).vesN=synaptosome_vesN(1, idxSynap);   % number of vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_pos=[];       % position of the vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_rad_max=[];       % largest axis of the vesicles in the actual synaptosome
        synaptosome(idxSynap).ves_rad=[];       % radii of the ellipsoidal vesicles in the actual synaptosome
        
        % loop throught the vesicles in the synaptosome
        idx_ves=1;      % index of the actual vesicle
        actVesTries_N=0;      % number of tries given in placing the actual vesicle (the vesicles can not overlap)
        while idx_ves<=synaptosome(idxSynap).vesN
            actVesTries_N=actVesTries_N+1;  % one more try was given in placing the actual vesicle
            
            % place the vesicle randomly in a cube determined by the
            % synaptosome radius, will discard it later, if it is outside
            % of the sphere
            actVesPos=(rand(3,1)-0.5)*2*synaptosome(idxSynap).rad+synaptosome(idxSynap).pos;
            
            % generate the radius for the actual vesicle
            actVesRad_max=R_mean_max+R_sig.*randn(1, 1);
            actVesRad=R_mean+R_sig.*randn(3, 1);
            
            % examine whether the actual vesicle is inside the actual
            % synaptosome
            synaptosome_withinBool=sum((actVesPos-synaptosome(idxSynap).pos).^2)<=(synaptosome(idxSynap).rad-actVesRad_max)^2;
            
            % examine whether the actual vesicle overlaps with any of the
            % previously placed vesicles in the synaptosome
            if ~isempty(synaptosome(idxSynap).ves_pos)      % if it is not the first vesicle in the synaptosome
                vesDistances=sum((synaptosome(idxSynap).ves_pos-repmat(actVesPos, [1, size(synaptosome(idxSynap).ves_pos,2)])).^2,1);   % square of distances of the actual and the previously placed vesicles
                vesRadSums=synaptosome(idxSynap).ves_rad_max+repmat(actVesRad_max, [1, size(synaptosome(idxSynap).ves_rad_max,2)]);         % sum of radii of the actual and the previously placed vesicles
                vesDistances_RadSums_Bool=vesDistances>=vesRadSums.^2;      % examine whether actual vesicle overlaps with any other in the synaptosome
                vesicle_distBool=prod(vesDistances_RadSums_Bool);       % true if the actual vesicle does not overlap with any other in the synaptosome
            else        % if it is the first vesicle in the synaptosome
                vesicle_distBool=true;
            end
            
            % if the actual vesicle is inside the synaptosome and does not
            % overlap with any of the previously placed vesicles, add it to
            % the parameters of the actual synaptosome and leap to the next
            % vesicle
            if synaptosome_withinBool && vesicle_distBool
                idx_ves=idx_ves+1;
                actVesTries_N=0;
                synaptosome(idxSynap).ves_pos=[synaptosome(idxSynap).ves_pos, actVesPos];
                synaptosome(idxSynap).ves_rad_max=[synaptosome(idxSynap).ves_rad_max, actVesRad_max];
                synaptosome(idxSynap).ves_rad=[synaptosome(idxSynap).ves_rad, actVesRad];
            end
            
            % if too many tries was given in placing the vesicle, consider
            % not any more vesicle can be placed in this synatosome, leap
            % to the next synaptosome
            if actVesTries_N>100
                disp('Cannot place any more vesicle in this synaptosome.')
                break;
            end
            
        end
    end
    
   %% vesicles in the solution
   
    ves_N_solution=params.sample.ves_rand_solution_N;  % number of vesicles
    ves_dim=params.sample.ves_rand_solution_area;      % "x" and "y" extension of area in which the vesicles are distributed
    ves_zmax=params.sample.ves_rand_zmax;    % "z" extension of area in which the vesicles are distributed
    
    solution.ves_pos=[];
    solution.ves_rad_max=[];
    solution.ves_rad=[];
    
    
    % loop throught the vesicles in the solution
    idx_ves=1;      % index of the actual vesicle
    actVesTries_N=0;      % number of tries given in placing the actual vesicle (the vesicles can not overlap)
    while idx_ves<=ves_N_solution
        actVesTries_N=actVesTries_N+1;  % one more try was given in placing the actual vesicle
        
        % place the vesicle randomly in a cube determined by the
        % "ves_dim" and "z_max" parameters, will discard it later, if it is outside
        % of the sphere
        actVesPos=[...
            (rand(1,1)-0.5)*ves_dim;...
            (rand(1,1)-0.5)*ves_dim;...
            (rand(1,1)-0.5)*ves_zmax];  % 1x(ves_fix_N/ves_N_ofves) array containing coordinates of vesicles
        
        % generate the radius for the actual vesicle
        actVesRad_max=R_mean_max+R_sig.*randn(1, 1);
        actVesRad=R_mean+R_sig.*randn(3, 1);
        
        % examine whether the actual vesicle is inside any of the
        % synaptosomes
        synaptosome_withinBool=false;
        for idxSynap=1:synaptosome_N
            if synaptosome_vesN(1, idxSynap)==0 % skip, if this synaptosome contains no vesicles
                synaptosome_withinBool=false;
            else
                synaptosome_withinBool=sum((actVesPos-synaptosome(idxSynap).pos).^2)<=(synaptosome(idxSynap).rad+actVesRad_max)^2;
            end
            if synaptosome_withinBool
                break;
            end
        end
        
        % examine whether the actual vesicle overlaps with any of the
        % previously placed vesicles in the solution
        if ~isempty(solution.ves_pos)      % if it is not the first vesicle in the solution
            vesDistances=sum((solution.ves_pos-repmat(actVesPos, [1, size(solution.ves_pos,2)])).^2,1);   % square of distances of the actual and the previously placed vesicles
            vesRadSums=solution.ves_rad_max+repmat(actVesRad_max, [1, size(solution.ves_rad_max,2)]);         % sum of radii of the actual and the previously placed vesicles
            vesDistances_RadSums_Bool=vesDistances>=vesRadSums.^2;      % examine whether actual vesicle overlaps with any other in the solution
            vesicle_distBool=prod(vesDistances_RadSums_Bool);       % true if the actual vesicle does not overlap with any other in the solution
        else        % if it is the first vesicle in the solution
            vesicle_distBool=true;
        end
        
        % if the actual vesicle is not inside any of the synaptosomes and does not
        % overlap with any of the previously placed vesicles, add it to
        % the parameters of the solution and leap to the next
        % vesicle
        if ~synaptosome_withinBool && vesicle_distBool
            idx_ves=idx_ves+1;
            actVesTries_N=0;
            solution.ves_pos=[solution.ves_pos, actVesPos];
            solution.ves_rad_max=[solution.ves_rad_max, actVesRad_max];
            solution.ves_rad=[solution.ves_rad, actVesRad];
        end
        
        % if too many tries was given in placing the vesicle, consider
        % not any more vesicle can be placed in this solution, leap
        % to the next synaptosome
        if actVesTries_N>100
            disp('Cannot place any more vesicle in the solution region.')
            break;
        end
        
    end
    
    
    %% combined vesicles
    
    % concatenate the positions of the vesicles in different synaptosomes
    vesicle_coords.pos=[synaptosome(:).ves_pos, solution.ves_pos];  % 3x(ves_fix_N/ves_N_ofves) array containing coordinates of vesicles
    
    ves_N_final=size(vesicle_coords.pos, 2);
    
    % the total number of vesicels
    vesicle_coords.ves_N=ves_N_final;
    
    % concatenate the radii of the vesicles in different synaptosomes
    vesicle_coords.rad=[synaptosome(:).ves_rad, solution.ves_rad];
    
    % rotation angles of the ellipsoid
    %theta_rot_ellipsoids=acos(1-2*rand(N_ves,1));     % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.theta=zeros(ves_N_final,1);      % angle for the ellipsoid's rotation around the "y" axis
    vesicle_coords.fi=2*pi*rand(ves_N_final,1);  % angle for the ellipsoid's rotation around the "z" axis
    %fi_rot_ellipsoids=zeros(N_ves,1);  % angle for the ellipsoid's rotation around the "z" axis
    
    
    
else
    error('Error in vesicles coordinates');
end

end

