function [w_0_x, w_0_y, z_R_x, z_R_y, z_focal_x, z_focal_y]=m_gen_PSF_Gaussian_beam_params(params)

astigmatic_bool=params.meas.astigmatic_bool;

NA=params.aq.NA;                    % objective's numerical aperture
n_i=params.aq.n_immersion;          % refractive index of the immersion medium
n_s=params.aq.n_sample;             % refractive index of the sample medium
lambda=params.dye.lambda*1E-9;           % peak emission wavelength of the dye in m

% calculating permanent beam diameters
% from Gerhard J Schtz, Single-Molecule Analysis of Biomembranes p. 24 and ...
% Accuracy of the Gaussian Point Spread Function model in 2D localization microscopy
if ~astigmatic_bool     % non-astigmatic PSF
    sigma_0_x=0.25*lambda/NA;   % standard deviation of the PSF in the waist in the "x" direction
    w_0_x=2*sigma_0_x;          % beam waist size in the "x" direction
    w_0_y=w_0_x;
    z_R_x=pi*w_0_x^2*n_s/lambda;	% Rayleigh length of the beam in the "x" direction
    z_R_y=z_R_x;
    z_focal_x=0;                % distance of the corrdinate system's origo from the focal point
    z_focal_y=z_focal_x;
    
elseif astigmatic_bool  % astigmatic PSF
    %Coordinates are calculated in the image space than converted into
    %object space coordiates. It is also supposed that the cylindrical lens
    %is located very close to the tube lens.
    focal_length_tube=params.meas.tube_f*1E-2;               % focal length of tube lens in m
    M_lateral=params.meas.magnification;                     % lateral magnification of hte microscope
    %focal_length_obj=n_i*focal_length_tube/M_lateral;      % focal length of the objective in nm in the immersion medium
    focal_length_cylindrical=params.meas.cylindrical_f;      % focal length of the cylindrical lens in m
    %focal_plane_dist=params.aq.focal_plane_dist*1E-6;       % distance between the focal plane and the objective's aperture in nm
    
    M_axial=-M_lateral^2/n_s;                              % axial magnification of the microscope, can be derived from the lens equations applied for the objective's ideal thin lens and the tube lens, differ from the formula of the axial magnification of a single lens with the refractive index
    
    focal_length_x=focal_length_tube;                      % focal length of the tube lens without the cylindrical lens
    focal_length_y=focal_length_x*focal_length_cylindrical/(focal_length_x+focal_length_cylindrical);   % focal length of the tube lens with the cylindrical lens
    
    focal_shift_y=-(focal_length_y-focal_length_x)/M_axial;    % difference between the two focal points in object space coordinates, "-" sign is needed because the "z" axis is inverted in object space campared to image space
    
    NA_x=NA;    % numerical aperture in the "x-z" plane
    alpha_x=asin(NA_x/n_i);         % approximately the maximal half-angle of the cone of light in the "x-z" plane
    %alpha_y=atan((tan(alpha_x)*focal_plane_dist)/(focal_plane_dist-focal_shift_y));     % approximately the maximal half-angle of the cone of light in the "y-z" plane
    alpha_y=alpha_x;                % approximately the maximal half-angle of the cone of light in the "y-z" plane
    NA_y=n_i*sin(alpha_y);          % numerical aperture in the "y-z" plane
    sigma_0_x=0.25*lambda/NA_x;     % standard deviation of the PSF in the waist in the "x-z" plane
    sigma_0_y=0.25*lambda/NA_y;     % standard deviation of the PSF in the waist in the "y-z" plane
    w_0_x=2*sigma_0_x;              % waist size in the "x-z" plane
    w_0_y=2*sigma_0_y;              % waist size in the "y-z" plane
    z_R_x=pi*w_0_x^2*n_s/lambda;        % Rayleigh length of the beam in the "x-z" plane as in the source
    z_R_y=pi*w_0_y^2*n_s/lambda;        % Rayleigh length of the beam in the "y-z" plane as in the source
    %z_R_x=pi*w_0_x^2*n_i/lambda;   % this should be the Rayleigh length, but the above gives the numerically more correct results, probably the PSF can not approximated with a Gaussian beam, Rayleigh length should also be derived from measurement and not from this simple theorical consideration....
    %z_R_y=pi*w_0_y^2*n_i/lambda;   % this should be the Rayleigh length, but the above gives the numerically more correct results, probably the PSF can not approximated with a Gaussian beam, Rayleigh length should also be derived from measurement and not from this simple theorical consideration....
    % calculating the "z" position where the beam has the same diameter in 
    % the "x-z" and "y-z" planes, solving a second-order equation instead
    % of approximating this position with the half distance between the two
    % focal points
    % this was solved: 0=w_x(0)^2-w_y(0)^2=w_0_x^2*(1+(0-z_focal_x)^2/z_R_x^2)-w_0_y^2*(1+(0-(focal_shift_y+z_focal_x))^2/z_R_y^2);
    % coeffitients in the second order equation
    a=(w_0_x/z_R_x)^2-(w_0_y/z_R_y)^2;
    b=-2*(w_0_y/z_R_y)^2*focal_shift_y;
    c=w_0_x^2-w_0_y^2-(w_0_y/z_R_y*focal_shift_y)^2;
    % solution of the second order equation
    if a==0
        z_focal_x=-c/b;                         % "z" coordinate of the focal point in the "x-z" plane (tube lens' focal point) in object space coordinates
        z_focal_y=focal_shift_y+z_focal_x;      % "z" coordinate of the focal point in the "y-z" plane (joint focal point of the tube and cylindrical lens) in object space coordinates
    else
        z_focal_x=(-b+sqrt(b^2-4*a*c))/(2*a);   % "z" coordinate of the focal point in the "x-z" plane (tube lens' focal point) in object space coordinates
        z_focal_y=focal_shift_y+z_focal_x;      % "z" coordinate of the focal point in the "y-z" plane (joint focal point of the tube and cylindrical lens) in object space coordinates
    end
    
else
    error('Something is wrong with the boolean variable of the astigmatic 3D measurement') 
end


end